//-----------------------------------------------------------------------
// File : AsdxShaderUtil.h
// Desc : Asura DirectX Shader Utility
// Copyright(c) Project Asura. All right resreved.
//-----------------------------------------------------------------------

//-----------------------------------------------------------------------
// History
//-----------------------------------------------------------------------
// <---> 2011/07/23 [Pocol] VK쐬
// <End>


//-----------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------
#include "asdxShaderUtil.h"
#include <d3dx11.h>
#include <d3dcompiler.h>
#include <cassert>


namespace Asura {
namespace Graphics {

/////////////////////////////////////////////////////////////////////////
// ShaderUtil class
/////////////////////////////////////////////////////////////////////////

//-----------------------------------------------------------------------
//! @brief t@CVF[_RpCiMB)
//! @param [in] filename VF[_̃t@C
//! @param [in] entryPoint Gg[|Cg
//! @param [in] shader VF[_vt@C
//! @param [out] ppBlobOut oCgR[hi[C^tF[Xւ̃|C^
//-----------------------------------------------------------------------
bool ShaderUtil::CompileFromFileA
(
    const char* filename,
    const char* entryPoint,
    const char* shaderModel,
    ID3DBlob** ppBlobOut 
)
{
    HRESULT hr = S_OK;
    bool result = true;

    DWORD dwShaderFlags = D3DCOMPILE_ENABLE_STRICTNESS;

#if defined(DEBUG) || defined(_DEBUG)
    dwShaderFlags |= D3DCOMPILE_DEBUG;
#endif

    ID3DBlob* pErrorBlob;

    // t@CRpC
    hr = D3DX11CompileFromFileA( 
        filename, 
        NULL,
        NULL,
        entryPoint,
        shaderModel,
        dwShaderFlags,
        0,
        NULL,
        ppBlobOut,
        &pErrorBlob,
        NULL );
    if ( FAILED( hr ) )
    {
        result = false;
        ELOG( "Error : D3DX11CompileFromFileA() Failed. filename = %s", filename );
        if ( pErrorBlob != NULL )
        {
            // G[bZ[Wo
            ELOG( "Error : %s", (char*)pErrorBlob->GetBufferPointer() );
         	OutputDebugStringA( (char*)pErrorBlob->GetBufferPointer() );
        }
    }

    SAFE_RELEASE( pErrorBlob );

    return result;
}

//-----------------------------------------------------------------------
//! @brief t@CVF[_RpC(Unicode)
//! @param [in] filename VF[_̃t@C
//! @param [in] entryPoint Gg[|Cg
//! @param [in] shader VF[_vt@C
//! @param [out] ppBlobOut oCgR[hi[C^tF[Xւ̃|C^
//-----------------------------------------------------------------------
bool ShaderUtil::CompileFromFileW
(
    const wchar_t* filename,
    const char* entryPoint,
    const char* shaderModel,
    ID3DBlob** ppBlobOut 
)
{
    HRESULT hr = S_OK;
    bool result = true;

    DWORD dwShaderFlags = D3DCOMPILE_ENABLE_STRICTNESS;

#if defined(DEBUG) || defined(_DEBUG)
    dwShaderFlags |= D3DCOMPILE_DEBUG;
#endif

    ID3DBlob* pErrorBlob;

    // t@CRpC
    hr = D3DX11CompileFromFileW( 
        filename, 
        NULL,
        NULL,
        entryPoint,
        shaderModel,
        dwShaderFlags,
        0,
        NULL,
        ppBlobOut,
        &pErrorBlob,
        NULL );
    if ( FAILED( hr ) )
    {
        result = false;
		printf( "[File: %s, Line: %d] ", __FILE__, __LINE__ );
		wprintf( L"Error : D3DX11CompileFromFileW() Failed. filename = %s\n", filename );
        if ( pErrorBlob != NULL )
        {
            // G[bZ[Wo
			ELOG( "Error : %s\n", (char*)pErrorBlob->GetBufferPointer() );
         	OutputDebugStringA( (char*)pErrorBlob->GetBufferPointer() );
        }
    }

    SAFE_RELEASE( pErrorBlob );

    return result;
}

//-----------------------------------------------------------------------
//! @brief t@CVF[_RpC
//! @param [in] filename VF[_̃t@C
//! @param [in] entryPoint Gg[|Cg
//! @param [in] shader VF[_vt@C
//! @param [out] ppBlobOut oCgR[hi[C^tF[Xւ̃|C^
//-----------------------------------------------------------------------
bool ShaderUtil::CompileFromFile
(
    const tchar* filename,
    const char* entryPoint,
    const char* shaderModel,
    ID3DBlob** ppBlobOut 
)
{
    HRESULT hr = S_OK;
    bool result = true;

    DWORD dwShaderFlags = D3DCOMPILE_ENABLE_STRICTNESS;

#if defined(DEBUG) || defined(_DEBUG)
    dwShaderFlags |= D3DCOMPILE_DEBUG;
#endif

    ID3DBlob* pErrorBlob;

    // t@CRpC
    hr = D3DX11CompileFromFile( 
        filename, 
        NULL,
        NULL,
        entryPoint,
        shaderModel,
        dwShaderFlags,
        0,
        NULL,
        ppBlobOut,
        &pErrorBlob,
        NULL );
    if ( FAILED( hr ) )
    {
        result = false;
#ifdef UNICODE
		printf( "[File: %s, Line: %d] ", __FILE__, __LINE__ );
		wprintf( L"Error : D3DX11CompileFromFile() Failed. filename = %s\n",filename );
#else
        ELOG( "Error : D3DX11CompileFromFile() Failed. filename = %s", filename );
#endif

        if ( pErrorBlob != NULL )
        {
            // G[bZ[Wo
            ELOG( "Error : %s", (char*)pErrorBlob->GetBufferPointer() );
         	OutputDebugStringA( (char*)pErrorBlob->GetBufferPointer() );
        }
    }

    SAFE_RELEASE( pErrorBlob );

    return result;
}

//-----------------------------------------------------------------------
//! @brief t@C璸_VF[_RpCāC
//! @param [in] filename VF[_̃t@C
//! @param [in] entryPoint Gg[|Cg
//! @param [in] shader VF[_vt@C
//! @param [in] pDevice D3D11foCXC^tF[X
//! @param [out] ppBlobOut oCgR[hi[ϐւ̃|C^
//! @param [out] ppShader ꂽVF[_i[ϐւ̃|C^
//-----------------------------------------------------------------------
bool ShaderUtil::CreateVertexShaderFromFileA
(
    const char* filename,
    const char* entryPoint,
    const char* shaderModel,
    ID3D11Device* pDevice,
    ID3DBlob** ppBlob,
    ID3D11VertexShader** ppShader 
)
{
    // t@CRpC
    if ( !CompileFromFileA( filename, entryPoint, shaderModel, ppBlob ) )
    {
        ELOG( "Error : CompileFromFileA() Failed. filename = %s", filename );
        SAFE_RELEASE( (*ppBlob) );
        return false;
    }

    // VF[_
    HRESULT hr = pDevice->CreateVertexShader( (*ppBlob)->GetBufferPointer(), (*ppBlob)->GetBufferSize(), NULL, ppShader );
    if ( FAILED( hr ) )
    {
        ELOG( "Error : ID3D11Device::CreateVertexShader() Failed. filename = %s", filename );
        SAFE_RELEASE( (*ppBlob) );
        return false;
    }

    return true;   
}


//-----------------------------------------------------------------------
//! @brief t@C璸_VF[_RpCāC
//! @param [in] filename VF[_̃t@C
//! @param [in] entryPoint Gg[|Cg
//! @param [in] shader VF[_vt@C
//! @param [in] pDevice D3D11foCXC^tF[X
//! @param [out] ppBlobOut oCgR[hi[ϐւ̃|C^
//! @param [out] ppShader ꂽVF[_i[ϐւ̃|C^
//-----------------------------------------------------------------------
bool ShaderUtil::CreateVertexShaderFromFileW
(
    const wchar_t* filename,
    const char* entryPoint,
    const char* shaderModel,
    ID3D11Device* pDevice,
    ID3DBlob** ppBlob,
    ID3D11VertexShader** ppShader 
)
{
    // t@CRpC
    if ( !CompileFromFileW( filename, entryPoint, shaderModel, ppBlob ) )
    {
        ELOG( "Error : CompileFromFileW() Failed. filename = %s", filename );
        SAFE_RELEASE( (*ppBlob) );
        return false;
    }

    // VF[_
    HRESULT hr = pDevice->CreateVertexShader( (*ppBlob)->GetBufferPointer(), (*ppBlob)->GetBufferSize(), NULL, ppShader );
    if ( FAILED( hr ) )
    {
        ELOG( "Error : ID3D11Device::CreateVertexShader() Failed. filename = %s", filename );
        SAFE_RELEASE( (*ppBlob) );
        return false;
    }

    return true;   
}


//-----------------------------------------------------------------------
//! @brief t@C璸_VF[_RpCāC
//! @param [in] filename VF[_̃t@C
//! @param [in] entryPoint Gg[|Cg
//! @param [in] shader VF[_vt@C
//! @param [in] pDevice D3D11foCXC^tF[X
//! @param [out] ppBlobOut oCgR[hi[ϐւ̃|C^
//! @param [out] ppShader ꂽVF[_i[ϐւ̃|C^
//-----------------------------------------------------------------------
bool ShaderUtil::CreateVertexShaderFromFile
(
    const tchar* filename,
    const char* entryPoint,
    const char* shaderModel,
    ID3D11Device* pDevice,
    ID3DBlob** ppBlob,
    ID3D11VertexShader** ppShader 
)
{
    // t@CRpC
    if ( !CompileFromFile( filename, entryPoint, shaderModel, ppBlob ) )
    {
        ELOG( "Error : CompileFromFile() Failed. filename = %s", filename );
        SAFE_RELEASE( (*ppBlob) );
        return false;
    }

    // VF[_
    HRESULT hr = pDevice->CreateVertexShader( (*ppBlob)->GetBufferPointer(), (*ppBlob)->GetBufferSize(), NULL, ppShader );
    if ( FAILED( hr ) )
    {
        ELOG( "Error : ID3D11Device::CreateVertexShader() Failed. filename = %s", filename );
        SAFE_RELEASE( (*ppBlob) );
        return false;
    }

    return true;   
}


//-----------------------------------------------------------------------
//! @brief t@CsNZVF[_RpCāC
//! @param [in] filename VF[_̃t@C
//! @param [in] entryPoint Gg[|Cg
//! @param [in] shader VF[_vt@C
//! @param [in] pDevice D3D11foCXC^tF[X
//! @param [out] ppShader ꂽVF[_i[ϐւ̃|C^
//-----------------------------------------------------------------------
bool ShaderUtil::CreatePixelShaderFromFileA
(
    const char* filename,
    const char* entryPoint,
    const char* shaderModel,
    ID3D11Device* pDevice,
    ID3D11PixelShader** ppShader 
)
{
    ID3DBlob* pBlob;

    // t@CRpC
    if ( !CompileFromFileA( filename, entryPoint, shaderModel, &pBlob ) )
    {
        ELOG( "Error : CompileFromFileA() Failed. filename = %s", filename );
        SAFE_RELEASE( pBlob );
        return false;
    }

    // VF[_
    HRESULT hr = pDevice->CreatePixelShader( pBlob->GetBufferPointer(), pBlob->GetBufferSize(), NULL, ppShader );
    if ( FAILED( hr ) )
    {
        ELOG( "Error : ID3D11Device::CreatePixelShader() Failed. filename = %s", filename );
        SAFE_RELEASE( pBlob );
        return false;
    }

    // 
    SAFE_RELEASE( pBlob );

    return true;   
}

//-----------------------------------------------------------------------
//! @brief t@CsNZVF[_RpCāC
//! @param [in] filename VF[_̃t@C
//! @param [in] entryPoint Gg[|Cg
//! @param [in] shader VF[_vt@C
//! @param [in] pDevice D3D11foCXC^tF[X
//! @param [out] ppShader ꂽVF[_i[ϐւ̃|C^
//-----------------------------------------------------------------------
bool ShaderUtil::CreatePixelShaderFromFileW
(
    const wchar_t* filename,
    const char* entryPoint,
    const char* shaderModel,
    ID3D11Device* pDevice,
    ID3D11PixelShader** ppShader 
)
{
    ID3DBlob* pBlob;

    // t@CRpC
    if ( !CompileFromFileW( filename, entryPoint, shaderModel, &pBlob ) )
    {
        ELOG( "Error : CompileFromFileW() Failed. filename = %s", filename );
        SAFE_RELEASE( pBlob );
        return false;
    }

    // VF[_
    HRESULT hr = pDevice->CreatePixelShader( pBlob->GetBufferPointer(), pBlob->GetBufferSize(), NULL, ppShader );
    if ( FAILED( hr ) )
    {
        ELOG( "Error : ID3D11Device::CreatePixelShader() Failed. filename = %s", filename );
        SAFE_RELEASE( pBlob );
        return false;
    }

    // 
    SAFE_RELEASE( pBlob );

    return true;   
}

//-----------------------------------------------------------------------
//! @brief t@CsNZVF[_RpCāC
//! @param [in] filename VF[_̃t@C
//! @param [in] entryPoint Gg[|Cg
//! @param [in] shader VF[_vt@C
//! @param [in] pDevice D3D11foCXC^tF[X
//! @param [out] ppShader ꂽVF[_i[ϐւ̃|C^
//-----------------------------------------------------------------------
bool ShaderUtil::CreatePixelShaderFromFile
(
    const tchar* filename,
    const char* entryPoint,
    const char* shaderModel,
    ID3D11Device* pDevice,
    ID3D11PixelShader** ppShader 
)
{
    ID3DBlob* pBlob;

    // t@CRpC
    if ( !CompileFromFile( filename, entryPoint, shaderModel, &pBlob ) )
    {
        ELOG( "Error : CompileFromFile() Failed. filename = %s", filename );
        SAFE_RELEASE( pBlob );
        return false;
    }

    // VF[_
    HRESULT hr = pDevice->CreatePixelShader( pBlob->GetBufferPointer(), pBlob->GetBufferSize(), NULL, ppShader );
    if ( FAILED( hr ) )
    {
        ELOG( "Error : ID3D11Device::CreatePixelShader() Failed. filename = %s", filename );
        SAFE_RELEASE( pBlob );
        return false;
    }

    // 
    SAFE_RELEASE( pBlob );

    return true;   
}

//-----------------------------------------------------------------------
//! @brief t@CWIgVF[_RpCāC
//! @param [in] filename VF[_̃t@C
//! @param [in] entryPoint Gg[|Cg
//! @param [in] shader VF[_vt@C
//! @param [in] pDevice D3D11foCXC^tF[X
//! @param [out] ppShader ꂽVF[_i[ϐւ̃|C^
//-----------------------------------------------------------------------
bool ShaderUtil::CreateGeometryShaderFromFileA
(
    const char* filename,
    const char* entryPoint,
    const char* shaderModel,
    ID3D11Device* pDevice,
    ID3D11GeometryShader** ppShader 
)
{
    ID3DBlob* pBlob;

    // t@CRpC
    if ( !CompileFromFileA( filename, entryPoint, shaderModel, &pBlob ) )
    {
        ELOG( "Error : CompileFromFileA() Failed. filename = %s", filename );
        SAFE_RELEASE( pBlob );
        return false;
    }

    // VF[_
    HRESULT hr = pDevice->CreateGeometryShader( pBlob->GetBufferPointer(), pBlob->GetBufferSize(), NULL, ppShader );
    if ( FAILED( hr ) )
    {
        ELOG( "Error : ID3D11Device::CreateGeometryShader() Failed. filename = %s", filename );
        SAFE_RELEASE( pBlob );
        return false;
    }

    // 
    SAFE_RELEASE( pBlob );

    return true;   
}

//-----------------------------------------------------------------------
//! @brief t@CWIgVF[_RpCāC
//! @param [in] filename VF[_̃t@C
//! @param [in] entryPoint Gg[|Cg
//! @param [in] shader VF[_vt@C
//! @param [in] pDevice D3D11foCXC^tF[X
//! @param [out] ppShader ꂽVF[_i[ϐւ̃|C^
//-----------------------------------------------------------------------
bool ShaderUtil::CreateGeometryShaderFromFileW
(
    const wchar_t* filename,
    const char* entryPoint,
    const char* shaderModel,
    ID3D11Device* pDevice,
    ID3D11GeometryShader** ppShader 
)
{
    ID3DBlob* pBlob;

    // t@CRpC
    if ( !CompileFromFileW( filename, entryPoint, shaderModel, &pBlob ) )
    {
        ELOG( "Error : CompileFromFileW() Failed. filename = %s", filename );
        SAFE_RELEASE( pBlob );
        return false;
    }

    // VF[_
    HRESULT hr = pDevice->CreateGeometryShader( pBlob->GetBufferPointer(), pBlob->GetBufferSize(), NULL, ppShader );
    if ( FAILED( hr ) )
    {
        ELOG( "Error : ID3D11Device::CreateGeometryShader() Failed. filename = %s", filename );
        SAFE_RELEASE( pBlob );
        return false;
    }

    // 
    SAFE_RELEASE( pBlob );

    return true;   
}

//-----------------------------------------------------------------------
//! @brief t@CWIgVF[_RpCāC
//! @param [in] filename VF[_̃t@C
//! @param [in] entryPoint Gg[|Cg
//! @param [in] shader VF[_vt@C
//! @param [in] pDevice D3D11foCXC^tF[X
//! @param [out] ppShader ꂽVF[_i[ϐւ̃|C^
//-----------------------------------------------------------------------
bool ShaderUtil::CreateGeometryShaderFromFile
(
    const tchar* filename,
    const char* entryPoint,
    const char* shaderModel,
    ID3D11Device* pDevice,
    ID3D11GeometryShader** ppShader 
)
{
    ID3DBlob* pBlob;

    // t@CRpC
    if ( !CompileFromFile( filename, entryPoint, shaderModel, &pBlob ) )
    {
        ELOG( "Error : CompileFromFile() Failed. filename = %s", filename );
        SAFE_RELEASE( pBlob );
        return false;
    }

    // VF[_
    HRESULT hr = pDevice->CreateGeometryShader( pBlob->GetBufferPointer(), pBlob->GetBufferSize(), NULL, ppShader );
    if ( FAILED( hr ) )
    {
        ELOG( "Error : ID3D11Device::CreateGeometryShader() Failed. filename = %s", filename );
        SAFE_RELEASE( pBlob );
        return false;
    }

    // 
    SAFE_RELEASE( pBlob );

    return true;   
}

//-----------------------------------------------------------------------
//! @brief t@CnVF[_RpCāC
//! @param [in] filename VF[_̃t@C
//! @param [in] entryPoint Gg[|Cg
//! @param [in] shader VF[_vt@C
//! @param [in] pDevice D3D11foCXC^tF[X
//! @param [out] ppShader ꂽVF[_i[ϐւ̃|C^
//-----------------------------------------------------------------------
bool ShaderUtil::CreateHullShaderFromFileA
(
    const char* filename,
    const char* entryPoint,
    const char* shaderModel,
    ID3D11Device* pDevice,
    ID3D11HullShader** ppShader 
)
{
    ID3DBlob* pBlob;

    // t@CRpC
    if ( !CompileFromFileA( filename, entryPoint, shaderModel, &pBlob ) )
    {
        ELOG( "Error : CompileFromFileA() Failed. filename = %s", filename );
        SAFE_RELEASE( pBlob );
        return false;
    }

    // VF[_
    HRESULT hr = pDevice->CreateHullShader( pBlob->GetBufferPointer(), pBlob->GetBufferSize(), NULL, ppShader );
    if ( FAILED( hr ) )
    {
        ELOG( "Error : ID3D11Device::CreateHullShader() Failed. filename = %s", filename );
        SAFE_RELEASE( pBlob );
        return false;
    }

    // 
    SAFE_RELEASE( pBlob );

    return true;   
}

//-----------------------------------------------------------------------
//! @brief t@CnVF[_RpCāC
//! @param [in] filename VF[_̃t@C
//! @param [in] entryPoint Gg[|Cg
//! @param [in] shader VF[_vt@C
//! @param [in] pDevice D3D11foCXC^tF[X
//! @param [out] ppShader ꂽVF[_i[ϐւ̃|C^
//-----------------------------------------------------------------------
bool ShaderUtil::CreateHullShaderFromFileW
(
    const wchar_t* filename,
    const char* entryPoint,
    const char* shaderModel,
    ID3D11Device* pDevice,
    ID3D11HullShader** ppShader 
)
{
    ID3DBlob* pBlob;

    // t@CRpC
    if ( !CompileFromFileW( filename, entryPoint, shaderModel, &pBlob ) )
    {
        ELOG( "Error : CompileFromFileW() Failed. filename = %s", filename );
        SAFE_RELEASE( pBlob );
        return false;
    }

    // VF[_
    HRESULT hr = pDevice->CreateHullShader( pBlob->GetBufferPointer(), pBlob->GetBufferSize(), NULL, ppShader );
    if ( FAILED( hr ) )
    {
        ELOG( "Error : ID3D11Device::CreateHullShader() Failed. filename = %s", filename );
        SAFE_RELEASE( pBlob );
        return false;
    }

    // 
    SAFE_RELEASE( pBlob );

    return true;   
}

//-----------------------------------------------------------------------
//! @brief t@CnVF[_RpCāC
//! @param [in] filename VF[_̃t@C
//! @param [in] entryPoint Gg[|Cg
//! @param [in] shader VF[_vt@C
//! @param [in] pDevice D3D11foCXC^tF[X
//! @param [out] ppShader ꂽVF[_i[ϐւ̃|C^
//-----------------------------------------------------------------------
bool ShaderUtil::CreateHullShaderFromFile
(
    const tchar* filename,
    const char* entryPoint,
    const char* shaderModel,
    ID3D11Device* pDevice,
    ID3D11HullShader** ppShader 
)
{
    ID3DBlob* pBlob;

    // t@CRpC
    if ( !CompileFromFile( filename, entryPoint, shaderModel, &pBlob ) )
    {
        ELOG( "Error : CompileFromFile() Failed. filename = %s", filename );
        SAFE_RELEASE( pBlob );
        return false;
    }

    // VF[_
    HRESULT hr = pDevice->CreateHullShader( pBlob->GetBufferPointer(), pBlob->GetBufferSize(), NULL, ppShader );
    if ( FAILED( hr ) )
    {
        ELOG( "Error : ID3D11Device::CreateHullShader() Failed. filename = %s", filename );
        SAFE_RELEASE( pBlob );
        return false;
    }

    // 
    SAFE_RELEASE( pBlob );

    return true;   
}

//-----------------------------------------------------------------------
//! @brief t@C̈VF[_RpCāC
//! @param [in] filename VF[_̃t@C
//! @param [in] entryPoint Gg[|Cg
//! @param [in] shader VF[_vt@C
//! @param [in] pDevice D3D11foCXC^tF[X
//! @param [out] ppShader ꂽVF[_i[ϐւ̃|C^
//-----------------------------------------------------------------------
bool ShaderUtil::CreateDomainShaderFromFileA
(
    const char* filename,
    const char* entryPoint,
    const char* shaderModel,
    ID3D11Device* pDevice,
    ID3D11DomainShader** ppShader 
)
{
    ID3DBlob* pBlob;

    // t@CRpC
    if ( !CompileFromFileA( filename, entryPoint, shaderModel, &pBlob ) )
    {
        ELOG( "Error : CompileFromFileA() Failed. filename = %s", filename );
        SAFE_RELEASE( pBlob );
        return false;
    }

    // VF[_
    HRESULT hr = pDevice->CreateDomainShader( pBlob->GetBufferPointer(), pBlob->GetBufferSize(), NULL, ppShader );
    if ( FAILED( hr ) )
    {
        ELOG( "Error : ID3D11Device::CreateDomainShader() Failed. filename = %s", filename );
        SAFE_RELEASE( pBlob );
        return false;
    }

    // 
    SAFE_RELEASE( pBlob );

    return true;   
}

//-----------------------------------------------------------------------
//! @brief t@C̈VF[_RpCāC
//! @param [in] filename VF[_̃t@C
//! @param [in] entryPoint Gg[|Cg
//! @param [in] shader VF[_vt@C
//! @param [in] pDevice D3D11foCXC^tF[X
//! @param [out] ppShader ꂽVF[_i[ϐւ̃|C^
//-----------------------------------------------------------------------
bool ShaderUtil::CreateDomainShaderFromFileW
(
    const wchar_t* filename,
    const char* entryPoint,
    const char* shaderModel,
    ID3D11Device* pDevice,
    ID3D11DomainShader** ppShader 
)
{
    ID3DBlob* pBlob;

    // t@CRpC
    if ( !CompileFromFileW( filename, entryPoint, shaderModel, &pBlob ) )
    {
        ELOG( "Error : CompileFromFileW() Failed. filename = %s", filename );
        SAFE_RELEASE( pBlob );
        return false;
    }

    // VF[_
    HRESULT hr = pDevice->CreateDomainShader( pBlob->GetBufferPointer(), pBlob->GetBufferSize(), NULL, ppShader );
    if ( FAILED( hr ) )
    {
        ELOG( "Error : ID3D11Device::CreateDomainShader() Failed. filename = %s", filename );
        SAFE_RELEASE( pBlob );
        return false;
    }

    // 
    SAFE_RELEASE( pBlob );

    return true;   
}

//-----------------------------------------------------------------------
//! @brief t@C̈VF[_RpCāC
//! @param [in] filename VF[_̃t@C
//! @param [in] entryPoint Gg[|Cg
//! @param [in] shader VF[_vt@C
//! @param [in] pDevice D3D11foCXC^tF[X
//! @param [out] ppShader ꂽVF[_i[ϐւ̃|C^
//-----------------------------------------------------------------------
bool ShaderUtil::CreateDomainShaderFromFile
(
    const tchar* filename,
    const char* entryPoint,
    const char* shaderModel,
    ID3D11Device* pDevice,
    ID3D11DomainShader** ppShader 
)
{
    ID3DBlob* pBlob;

    // t@CRpC
    if ( !CompileFromFile( filename, entryPoint, shaderModel, &pBlob ) )
    {
        ELOG( "Error : CompileFromFile() Failed. filename = %s", filename );
        SAFE_RELEASE( pBlob );
        return false;
    }

    // VF[_
    HRESULT hr = pDevice->CreateDomainShader( pBlob->GetBufferPointer(), pBlob->GetBufferSize(), NULL, ppShader );
    if ( FAILED( hr ) )
    {
        ELOG( "Error : ID3D11Device::CreateDomainShader() Failed. filename = %s", filename );
        SAFE_RELEASE( pBlob );
        return false;
    }

    // 
    SAFE_RELEASE( pBlob );

    return true;   
}

//-----------------------------------------------------------------------
//! @brief t@C牉ZVF[_RpCāC
//! @param [in] filename VF[_̃t@C
//! @param [in] entryPoint Gg[|Cg
//! @param [in] shader VF[_vt@C
//! @param [in] pDevice D3D11foCXC^tF[X
//! @param [out] ppShader ꂽVF[_i[ϐւ̃|C^
//-----------------------------------------------------------------------
bool ShaderUtil::CreateComputeShaderFromFileA
(
    const char* filename,
    const char* entryPoint,
    const char* shaderModel,
    ID3D11Device* pDevice,
    ID3D11ComputeShader** ppShader 
)
{
    ID3DBlob* pBlob;

    // t@CRpC
    if ( !CompileFromFileA( filename, entryPoint, shaderModel, &pBlob ) )
    {
        ELOG( "Error : CompileFromFileA() Failed. filename = %s", filename );
        SAFE_RELEASE( pBlob );
        return false;
    }

    // VF[_
    HRESULT hr = pDevice->CreateComputeShader( pBlob->GetBufferPointer(), pBlob->GetBufferSize(), NULL, ppShader );
    if ( FAILED( hr ) )
    {
        ELOG( "Error : ID3D11Device::CreateComputeShader() Failed. filename = %s", filename );
        SAFE_RELEASE( pBlob );
        return false;
    }

    // 
    SAFE_RELEASE( pBlob );

    return true;   
}

//-----------------------------------------------------------------------
//! @brief t@C牉ZVF[_RpCāC
//! @param [in] filename VF[_̃t@C
//! @param [in] entryPoint Gg[|Cg
//! @param [in] shader VF[_vt@C
//! @param [in] pDevice D3D11foCXC^tF[X
//! @param [out] ppShader ꂽVF[_i[ϐւ̃|C^
//-----------------------------------------------------------------------
bool ShaderUtil::CreateComputeShaderFromFileW
(
    const wchar_t* filename,
    const char* entryPoint,
    const char* shaderModel,
    ID3D11Device* pDevice,
    ID3D11ComputeShader** ppShader 
)
{
    ID3DBlob* pBlob;

    // t@CRpC
    if ( !CompileFromFileW( filename, entryPoint, shaderModel, &pBlob ) )
    {
        ELOG( "Error : CompileFromFileW() Failed. filename = %s", filename );
        SAFE_RELEASE( pBlob );
        return false;
    }

    // VF[_
    HRESULT hr = pDevice->CreateComputeShader( pBlob->GetBufferPointer(), pBlob->GetBufferSize(), NULL, ppShader );
    if ( FAILED( hr ) )
    {
        ELOG( "Error : ID3D11Device::CreateComputeShader() Failed. filename = %s", filename );
        SAFE_RELEASE( pBlob );
        return false;
    }

    // 
    SAFE_RELEASE( pBlob );

    return true;   
}

//-----------------------------------------------------------------------
//! @brief t@C牉ZVF[_RpCāC
//! @param [in] filename VF[_̃t@C
//! @param [in] entryPoint Gg[|Cg
//! @param [in] shader VF[_vt@C
//! @param [in] pDevice D3D11foCXC^tF[X
//! @param [out] ppShader ꂽVF[_i[ϐւ̃|C^
//-----------------------------------------------------------------------
bool ShaderUtil::CreateComputeShaderFromFile
(
    const tchar* filename,
    const char* entryPoint,
    const char* shaderModel,
    ID3D11Device* pDevice,
    ID3D11ComputeShader** ppShader 
)
{
    ID3DBlob* pBlob;

    // t@CRpC
    if ( !CompileFromFile( filename, entryPoint, shaderModel, &pBlob ) )
    {
        ELOG( "Error : CompileFromFile() Failed. filename = %s", filename );
        SAFE_RELEASE( pBlob );
        return false;
    }

    // VF[_
    HRESULT hr = pDevice->CreateComputeShader( pBlob->GetBufferPointer(), pBlob->GetBufferSize(), NULL, ppShader );
    if ( FAILED( hr ) )
    {
        ELOG( "Error : ID3D11Device::CreateComputeShader() Failed. filename = %s", filename );
        SAFE_RELEASE( pBlob );
        return false;
    }

    // 
    SAFE_RELEASE( pBlob );

    return true;   
}

//////////////////////////////////////////////////////////////////////////
// ShaderContainer class
//////////////////////////////////////////////////////////////////////////

//------------------------------------------------------------------------
//! @brief lݒ
//------------------------------------------------------------------------
ShaderContainer::ShaderContainer()
{
	mVSBlob.clear();
	mVS.clear();
	mPS.clear();
	mGS.clear();
	mHS.clear();
	mDS.clear();
	mCS.clear();
}

//------------------------------------------------------------------------
//! @brief j
//------------------------------------------------------------------------
ShaderContainer::~ShaderContainer()
{
	/* DO_NOTHING */
}

//------------------------------------------------------------------------
//! @brief 
//------------------------------------------------------------------------
void ShaderContainer::Release()
{
	for( uint32_t i=0; i<mVSBlob.size(); ++i )
	{ SAFE_RELEASE( mVSBlob[ i ] ); }

	for( uint32_t i=0; i<mVS.size(); ++i )
	{ SAFE_RELEASE( mVS[ i ] ); }

	for( uint32_t i=0; i<mPS.size(); ++i )
	{ SAFE_RELEASE( mPS[ i ] ); }

	for( uint32_t i=0; i<mGS.size(); ++i )
	{ SAFE_RELEASE( mGS[ i ] ); }

	for( uint32_t i=0; i<mHS.size(); ++i )
	{ SAFE_RELEASE( mHS[ i ] ); }

	for( uint32_t i=0; i<mDS.size(); ++i )
	{ SAFE_RELEASE( mDS[ i ] ); }

	for( uint32_t i=0; i<mCS.size(); ++i )
	{ SAFE_RELEASE( mCS[ i ] ); }

	mVSBlob.clear();
	mVS.clear();
	mPS.clear();
	mGS.clear();
	mHS.clear();
	mDS.clear();
	mCS.clear();
}

//------------------------------------------------------------------------
//! @brief t@CVF[_𐶐
//------------------------------------------------------------------------
bool ShaderContainer::CreateFromFileW
(
	ID3D11Device      *pDevice,
	wchar_t			  *pFilename,
	ShaderCompileDesc *pDescs,
	const int		   numDescs 
)
{
	bool isSuccess = false;

	for( int i=0; i<numDescs; ++i )
	{
		switch( pDescs[ i ].shaderType )
		{
		case SHADER_TYPE_VERTEX:
			{
				ID3DBlob *pVSBlob = null;
				ID3D11VertexShader* pVS = null;
				isSuccess = ShaderUtil::CreateVertexShaderFromFileW(
					pFilename,
					pDescs[ i ].pEntryPoint,
					pDescs[ i ].pShaderModel,
					pDevice,
					&pVSBlob,
					&pVS );

				mVSBlob.push_back( pVSBlob );
				mVS.push_back( pVS );
			}
			break;

		case SHADER_TYPE_PIXEL:
			{
				ID3D11PixelShader* pPS = null;
				isSuccess = ShaderUtil::CreatePixelShaderFromFileW(
					pFilename,
					pDescs[ i ].pEntryPoint,
					pDescs[ i ].pShaderModel,
					pDevice,
					&pPS );

				mPS.push_back( pPS );
			}
			break;

		case SHADER_TYPE_GEOMETRY:
			{
				ID3D11GeometryShader* pGS = null;
				isSuccess = ShaderUtil::CreateGeometryShaderFromFileW(
					pFilename,
					pDescs[ i ].pEntryPoint,
					pDescs[ i ].pShaderModel,
					pDevice,
					&pGS );

				mGS.push_back( pGS );
			}
			break;

		case SHADER_TYPE_HULL:
			{
				ID3D11HullShader* pHS = null;
				isSuccess = ShaderUtil::CreateHullShaderFromFileW(
					pFilename,
					pDescs[ i ].pEntryPoint,
					pDescs[ i ].pShaderModel,
					pDevice,
					&pHS );

				mHS.push_back( pHS );
			}
			break;

		case SHADER_TYPE_DOMAIN:
			{
				ID3D11DomainShader* pDS = null;
				isSuccess = ShaderUtil::CreateDomainShaderFromFileW(
					pFilename,
					pDescs[ i ].pEntryPoint,
					pDescs[ i ].pShaderModel,
					pDevice,
					&pDS );

				mDS.push_back( pDS );
			}
			break;

		case SHADER_TYPE_COMPUTE:
			{
				ID3D11ComputeShader* pCS = null;
				isSuccess = ShaderUtil::CreateComputeShaderFromFileW(
					pFilename,
					pDescs[ i ].pEntryPoint,
					pDescs[ i ].pShaderModel,
					pDevice,
					&pCS );

				mCS.push_back( pCS );
			}
			break;

		default:
			{
				/* DO_NOTHING */
			}
			break;
		}

		if ( !isSuccess )
		{
			ELOG( "Error : Shader Compile Failed. index = %d", i );
			Release();
			break;
		}
	}

	return isSuccess;
}

//------------------------------------------------------------------------
//! @brief t@CVF[_𐶐
//------------------------------------------------------------------------
bool ShaderContainer::CreateFromFileA
(
	ID3D11Device      *pDevice,
	char			  *pFilename,
	ShaderCompileDesc *pDescs,
	const int		   numDescs 
)
{
	bool isSuccess = false;

	for( int i=0; i<numDescs; ++i )
	{
		switch( pDescs[ i ].shaderType )
		{
		case SHADER_TYPE_VERTEX:
			{
				ID3DBlob *pVSBlob = null;
				ID3D11VertexShader* pVS = null;
				isSuccess = ShaderUtil::CreateVertexShaderFromFileA(
					pFilename,
					pDescs[ i ].pEntryPoint,
					pDescs[ i ].pShaderModel,
					pDevice,
					&pVSBlob,
					&pVS );

				mVSBlob.push_back( pVSBlob );
				mVS.push_back( pVS );
			}
			break;

		case SHADER_TYPE_PIXEL:
			{
				ID3D11PixelShader* pPS = null;
				isSuccess = ShaderUtil::CreatePixelShaderFromFileA(
					pFilename,
					pDescs[ i ].pEntryPoint,
					pDescs[ i ].pShaderModel,
					pDevice,
					&pPS );

				mPS.push_back( pPS );
			}
			break;

		case SHADER_TYPE_GEOMETRY:
			{
				ID3D11GeometryShader* pGS = null;
				isSuccess = ShaderUtil::CreateGeometryShaderFromFileA(
					pFilename,
					pDescs[ i ].pEntryPoint,
					pDescs[ i ].pShaderModel,
					pDevice,
					&pGS );

				mGS.push_back( pGS );
			}
			break;

		case SHADER_TYPE_HULL:
			{
				ID3D11HullShader* pHS = null;
				isSuccess = ShaderUtil::CreateHullShaderFromFileA(
					pFilename,
					pDescs[ i ].pEntryPoint,
					pDescs[ i ].pShaderModel,
					pDevice,
					&pHS );

				mHS.push_back( pHS );
			}
			break;

		case SHADER_TYPE_DOMAIN:
			{
				ID3D11DomainShader* pDS = null;
				isSuccess = ShaderUtil::CreateDomainShaderFromFileA(
					pFilename,
					pDescs[ i ].pEntryPoint,
					pDescs[ i ].pShaderModel,
					pDevice,
					&pDS );

				mDS.push_back( pDS );
			}
			break;

		case SHADER_TYPE_COMPUTE:
			{
				ID3D11ComputeShader* pCS = null;
				isSuccess = ShaderUtil::CreateComputeShaderFromFileA(
					pFilename,
					pDescs[ i ].pEntryPoint,
					pDescs[ i ].pShaderModel,
					pDevice,
					&pCS );

				mCS.push_back( pCS );
			}
			break;

		default:
			{
				/* DO_NOTHING */
			}
			break;
		}

		if ( !isSuccess )
		{
			ELOG( "Error : Shader Compile Failed. index = %d", i );
			Release();
			break;
		}
	}

	return isSuccess;
}

//------------------------------------------------------------------------
//! @brief t@CVF[_𐶐
//------------------------------------------------------------------------
bool ShaderContainer::CreateFromFile
(
	ID3D11Device      *pDevice,
	tchar			  *pFilename,
	ShaderCompileDesc *pDescs,
	const int		   numDescs 
)
{
	bool isSuccess = false;

	for( int i=0; i<numDescs; ++i )
	{
		switch( pDescs[ i ].shaderType )
		{
		case SHADER_TYPE_VERTEX:
			{
				ID3DBlob *pVSBlob = null;
				ID3D11VertexShader* pVS = null;
				isSuccess = ShaderUtil::CreateVertexShaderFromFile(
					pFilename,
					pDescs[ i ].pEntryPoint,
					pDescs[ i ].pShaderModel,
					pDevice,
					&pVSBlob,
					&pVS );

				mVSBlob.push_back( pVSBlob );
				mVS.push_back( pVS );
			}
			break;

		case SHADER_TYPE_PIXEL:
			{
				ID3D11PixelShader* pPS = null;
				isSuccess = ShaderUtil::CreatePixelShaderFromFile(
					pFilename,
					pDescs[ i ].pEntryPoint,
					pDescs[ i ].pShaderModel,
					pDevice,
					&pPS );

				mPS.push_back( pPS );
			}
			break;

		case SHADER_TYPE_GEOMETRY:
			{
				ID3D11GeometryShader* pGS = null;
				isSuccess = ShaderUtil::CreateGeometryShaderFromFile(
					pFilename,
					pDescs[ i ].pEntryPoint,
					pDescs[ i ].pShaderModel,
					pDevice,
					&pGS );

				mGS.push_back( pGS );
			}
			break;

		case SHADER_TYPE_HULL:
			{
				ID3D11HullShader* pHS = null;
				isSuccess = ShaderUtil::CreateHullShaderFromFile(
					pFilename,
					pDescs[ i ].pEntryPoint,
					pDescs[ i ].pShaderModel,
					pDevice,
					&pHS );

				mHS.push_back( pHS );
			}
			break;

		case SHADER_TYPE_DOMAIN:
			{
				ID3D11DomainShader* pDS = null;
				isSuccess = ShaderUtil::CreateDomainShaderFromFile(
					pFilename,
					pDescs[ i ].pEntryPoint,
					pDescs[ i ].pShaderModel,
					pDevice,
					&pDS );

				mDS.push_back( pDS );
			}
			break;

		case SHADER_TYPE_COMPUTE:
			{
				ID3D11ComputeShader* pCS = null;
				isSuccess = ShaderUtil::CreateComputeShaderFromFile(
					pFilename,
					pDescs[ i ].pEntryPoint,
					pDescs[ i ].pShaderModel,
					pDevice,
					&pCS );

				mCS.push_back( pCS );
			}
			break;

		default:
			{
				/* DO_NOTHING */
			}
			break;
		}

		if ( !isSuccess )
		{
			ELOG( "Error : Shader Compile Failed. index = %d", i );
			Release();
			break;
		}
	}

	return isSuccess;
}

//------------------------------------------------------------------------
//! @brief VF[_ݒ肷
//------------------------------------------------------------------------
void ShaderContainer::SetShader
(
	ID3D11DeviceContext* pImmediateContext,
	int indexVS,
	int indexPS,
	int indexGS,
	int indexHS,
	int indexDS )
{
	assert( pImmediateContext != NULL );
	assert( indexVS < (int)mVS.size() );
	assert( indexPS < (int)mPS.size() );
	assert( indexGS < (int)mGS.size() );
	assert( indexHS < (int)mHS.size() );
	assert( indexDS < (int)mDS.size() );

	ID3D11VertexShader*   pVS = ( indexVS >= 0 ) ? mVS[ indexVS ] : null;
	ID3D11PixelShader*    pPS = ( indexPS >= 0 ) ? mPS[ indexPS ] : null;
	ID3D11GeometryShader* pGS = ( indexGS >= 0 ) ? mGS[ indexGS ] : null;
	ID3D11HullShader*     pHS = ( indexHS >= 0 ) ? mHS[ indexHS ] : null;
	ID3D11DomainShader*   pDS = ( indexDS >= 0 ) ? mDS[ indexDS ] : null;

	pImmediateContext->VSSetShader( pVS, NULL, 0 );
	pImmediateContext->PSSetShader( pPS, NULL, 0 );
	pImmediateContext->GSSetShader( pGS, NULL, 0 );
	pImmediateContext->DSSetShader( pDS, NULL, 0 );
	pImmediateContext->HSSetShader( pHS, NULL, 0 );
}

//------------------------------------------------------------------------
//! @brief ZVF[_ݒ肷
//------------------------------------------------------------------------
void ShaderContainer::SetComputeShader
(
    ID3D11DeviceContext* pImmediateContext,
    int indexCS 
)
{
    assert( pImmediateContext != NULL );
    assert( indexCS < (int)mCS.size() );

    ID3D11ComputeShader* pCS = ( indexCS >= 0 ) ? mCS[ indexCS ] : null;

    pImmediateContext->CSSetShader( pCS, NULL, 0 );
}



} // namespace Graphics
} // namespace Asura