//--------------------------------------------------------------------
// File : Phong.hlsl
// Desc : Phong Model
// Copyright(c) Project Asura. All right reserved.
//--------------------------------------------------------------------

//--------------------------------------------------------------------
// Constant Buffer and Global Variables
//--------------------------------------------------------------------

//!< sobt@
cbuffer VSMatrix : register( b0 )
{
    float4x4 World : packoffset( c0 );		//!< [hs
    float4x4 View  : packoffset( c4 );		//!< r[s
    float4x4 Proj  : packoffset( c8 );		//!< ˉes
};

//!< }eAobt@
cbuffer PSMaterial : register( b1 )
{
    float3 Diffuse : packoffset( c0 );		//!< gUːF
    float  Alpha   : packoffset( c0.w );	//!< sx
    float3 Specular: packoffset( c1 );		//!< ʔːF
    float  Power   : packoffset( c1.w );	//!< ʔˋx
    float3 Emissive: packoffset( c2 );		//!< ȏƖF
};

//!< xNgobt@
cbuffer PSVector : register( b2 )
{
    float3 LightDir     : packoffset( c0 );	//!< CgxNg
    float3 ViewDir      : packoffset( c1 );	//!< xNg
};

//!< CgJ[obt@
cbuffer PSLightColor : register( b3 )
{
    float3 LightAmbient  : packoffset( c0 );	//!< 
    float3 LightDiffuse  : packoffset( c1 );	//!< gUˌ
	float3 LightSpecular : packoffset( c2 );	//!< ʔˌ
};


///////////////////////////////////////////////////////////////////////
// VSInput structure
///////////////////////////////////////////////////////////////////////
struct VSInput
{
    float3 Position : POSITION;		//!< ʒuW
    float3 Normal   : NORMAL;		//!< @xNg
    float2 TexCoord : TEXCOORD;		//!< eNX`W
};


///////////////////////////////////////////////////////////////////////
// VSOutput structure
///////////////////////////////////////////////////////////////////////
struct VSOutput
{
    float4 Position : SV_POSITION;	//!< ʒuW
    float2 TexCoord : TEXCOORD0;	//!< eNX`W
    float3 Normal   : TEXCOORD1;	//!< @xNg
};


//---------------------------------------------------------------------
//! @brief _VF[_CGg[|Cg
//! @param [in] input _̓f[^
//! @return _o̓f[^
//---------------------------------------------------------------------
VSOutput VSFunc( VSInput input )
{
    VSOutput output = (VSOutput)0;

    float4 localPos = float4( input.Position, 1.0f );
    float4 worldPos = mul( World, localPos );
    float4 viewPos  = mul( View, worldPos );
    float4 projPos  = mul( Proj, viewPos );

    output.Position = projPos;
    output.TexCoord = input.TexCoord;
    output.Normal   = input.Normal;

    return output;
}

//---------------------------------------------------------------------
//! @brief K}ϊs
//! @param [in] fLinear `l
//! @return K}2.2ɕϊlԋp
//---------------------------------------------------------------------
float4 Gamma( float4 fLinear )
{
	return pow( fLinear, 2.2f );
}

//---------------------------------------------------------------------
//! @brief PhongfŃCeBO
//! @param [in] N @xNg
//! @param [in] L CgxNg
//! @param [in] V xNg
//! @param [in] Kd fBt[YAxh
//! @param [in] Ks XyL[Axh
//! @param [in] n ʔˋx
//---------------------------------------------------------------------
float3 Phong
(
    float3 N,
    float3 L,
    float3 V,
    float3 Kd,
    float3 Ks,
    float  n
)
{
	float3 R = normalize( reflect( N, V ) );
	float3 cosAr = saturate( dot( V, N ) );
	float3 cosAl = saturate( dot( N, L ) );
	float3 C = ( Kd + Ks * ( n + 2 ) * 0.5f * pow( cosAr, n ) ) * cosAl;

	return C;
}

//---------------------------------------------------------------------
//! @brief sNZVF[_CGg[|Cg
//! @param [in] input _o̓f[^
//! @return sNZo̓f[^
//---------------------------------------------------------------------
float4 PSFunc( VSOutput input ) : SV_TARGET
{
    float4 output = (float4)0;

    // K
    float3 L = normalize( LightDir );
    float3 N = normalize( input.Normal );
	float3 V = normalize( -ViewDir );

	float3 Kd = LightDiffuse * Diffuse;
	float3 Ks = LightSpecular * Specular;

    // PhongCeBO
	output.rgb = Phong( N, L, V, Kd, Ks, Power );
	output.a = Alpha;
    
    return saturate( output );
}