//-----------------------------------------------------------------------
// File : SampleRenderer.cpp
// Desc : Sample of Renderer implementation.
// Copyright(c) Project Asura. All right reserved.
//-----------------------------------------------------------------------

//-----------------------------------------------------------------------
// History
//-----------------------------------------------------------------------
// <---> 2011/08/14 [Pocol] VK쐬
// <End>


//-----------------------------------------------------------------------
// Includes
//-----------------------------------------------------------------------
#include "SampleRenderer.h"
#include "asdxShaderUtil.h"


//-----------------------------------------------------------------------
// Using Statements
//-----------------------------------------------------------------------
using namespace Asura;
using namespace Asura::SampleProgram;
using namespace Asura::Graphics;
using namespace Asura::Math;
using namespace Asura::Util;


namespace Asura {
namespace SampleProgram {

//////////////////////////////////////////////////////////////////////////
// LightColorBuffer structure
//////////////////////////////////////////////////////////////////////////
typedef struct _LightColorBuffer
{
    AS_ALIGN16 D3DXVECTOR3 LightAmbient;    //!< 
    AS_ALIGN16 D3DXVECTOR3 LightDiffuse;    //!< gUˌ
	AS_ALIGN16 D3DXVECTOR3 LightSpecular;	//!< ʔˌ
} LightColorBuffer;


/////////////////////////////////////////////////////////////////////////
// SampleRenderer class
/////////////////////////////////////////////////////////////////////////

//-----------------------------------------------------------------------
//! @brief lݒ
//-----------------------------------------------------------------------
SampleRenderer::SampleRenderer()
{
	D3DXMatrixIdentity( &mWorld );
	D3DXMatrixIdentity( &mView );
	D3DXMatrixIdentity( &mProjection );

	mViewDir  = D3DXVECTOR3( 0.0f, 0.0f, 1.0f );
	mLightDir = D3DXVECTOR3( 0.0f, 1.0f, 0.0f );
}

//-----------------------------------------------------------------------
//! @brief j
//-----------------------------------------------------------------------
SampleRenderer::~SampleRenderer()
{
	/* DO_NOTHING */
}

//-----------------------------------------------------------------------
//! @brief 
//-----------------------------------------------------------------------
bool SampleRenderer::OnInit()
{
	// Jݒ
	D3DXVECTOR3 cameraPosition = D3DXVECTOR3( 0.0f, 2.5f, -5.0f );
	D3DXVECTOR3 cameraTarget   = D3DXVECTOR3( 0.0f, 0.0f, 0.0f );
	D3DXVECTOR3 cameraUpVector = D3DXVECTOR3( 0.0f, 1.0f, 0.0f );

	// svZ
	D3DXMatrixIdentity( &mWorld );
	D3DXMatrixLookAtLH( &mView, &cameraPosition, &cameraTarget, &cameraUpVector );
	D3DXMatrixPerspectiveFovLH( &mProjection, Math::PiOver4, DemoApp::GetInstance()->GetAspectRatio(), 1.0f, 1000.0f );

	// xNgݒ
	mViewDir = cameraTarget - cameraPosition;
	mLightDir = D3DXVECTOR3( 0.0f, 1.0f, -0.5f );
    mLightAmbient  = D3DXVECTOR3( 0.075f, 0.075f, 0.075f );
    mLightDiffuse  = D3DXVECTOR3( 0.85f, 0.85f, 0.85f );
	mLightSpecular = D3DXVECTOR3( 0.8f, 0.8f, 0.8f );

	// f[h
	if ( !mSaturn.LoadFromX( "Media/Models/saturn.x" ) )
	{
		ELOG( "Error : Model Load Failed." );
		return false;
	}

	if ( !mGround.LoadFromX( "Media/Models/ground.x" ) )
	{
		ELOG( "Error : Model Load Failed." );
		return false;
	}

	return true;
}

//-----------------------------------------------------------------------
//! @brief 
//-----------------------------------------------------------------------
void SampleRenderer::OnRelease()
{
	mSaturn.Release();
	mGround.Release();
	mShaders.Release();
	ModelUtil::Release();
    SAFE_RELEASE( mpLightColorBuffer );
}

//-----------------------------------------------------------------------
//! @brief foCX̏
//-----------------------------------------------------------------------
bool SampleRenderer::OnCreateDevice
(
	ID3D11Device *pDevice,
	const DXGI_SURFACE_DESC *pDesc
)
{
	HRESULT hr = S_OK;

	// f[eBeB[
	if ( !ModelUtil::Initialize( pDevice ) )
	{
		ELOG( "Error : ModelUitl::Iniatialize() Failed." );
		return false;
	}
	
	// f̃obt@𐶐
	if ( !mSaturn.CreateBuffer( pDevice ) )
	{
		ELOG( "Error : ModelUtil::CreateBuffer() Failed." );
		return false;
	}
	if ( !mGround.CreateBuffer( pDevice ) )
	{
		ELOG( "Error : ModelUtil::CreateBuffer() Failed." );
		return false;
	}

	// VF[_̃RpCݒ
	ShaderCompileDesc shaderCompileDescs[] = {
		{ SHADER_TYPE_VERTEX, "VSFunc", "vs_4_0" },
   		{ SHADER_TYPE_PIXEL,  "PSFunc", "ps_4_0" },
	};
	const uint32_t numShaderCompileDescs = sizeof( shaderCompileDescs ) / sizeof( shaderCompileDescs[ 0 ] );

	// t@CVF[_RpC
   	if ( !mShaders.CreateFromFile( 
		pDevice,
		TEXT( "Media/Shaders/Phong.hlsl" ),
		shaderCompileDescs,
		numShaderCompileDescs ) )
	{
		ELOG( "Error : ShaderContainer::CreateFromFile() Failed." );
		return false;
	}

    // xNgobt@̐ݒ
    D3D11_BUFFER_DESC bufDesc;
    memset( &bufDesc, 0, sizeof( bufDesc ) );
    bufDesc.Usage = D3D11_USAGE_DEFAULT;
    bufDesc.ByteWidth = sizeof( LightColorBuffer );
    bufDesc.BindFlags = D3D11_BIND_CONSTANT_BUFFER;
    bufDesc.CPUAccessFlags = 0;

	// xNgobt@̐
    hr = pDevice->CreateBuffer( &bufDesc, NULL, &mpLightColorBuffer );
    if ( FAILED( hr ) )
    {
        ELOG( "Error : ID3D11Device::CreateBuffer() Failed." );
        ELOG( "Error : Constant Buffer for Vector Create Failed" );
        return false;
    }
	
	return true;
}

//-----------------------------------------------------------------------
//! @brief foCXj̏
//-----------------------------------------------------------------------
void SampleRenderer::OnDestroyDevice()
{
	mSaturn.ReleaseBuffer();
	mGround.ReleaseBuffer();
	mShaders.Release();
	ModelUtil::Release();
    SAFE_RELEASE( mpLightColorBuffer );
}

//-----------------------------------------------------------------------
//! @brief Xbv`FC̃TCY̏
//-----------------------------------------------------------------------
bool SampleRenderer::OnResizeSwapChain
(
	ID3D11Device *pDeivce,
	IDXGISwapChain *pSwapChain,
	const DXGI_SURFACE_DESC *pDesc 
)
{
	D3DXMatrixPerspectiveFovLH( &mProjection, Math::PiOver4, DemoApp::GetInstance()->GetAspectRatio(), 1.0f, 1000.0f );
	return true;
}

//-----------------------------------------------------------------------
//! @brief XV
//-----------------------------------------------------------------------
void SampleRenderer::OnUpdate( double time, float elapsedTime )
{
    DemoApp::GetInstance()->GetCamera()->Compute();
}

//-----------------------------------------------------------------------
//! @brief `揈
//-----------------------------------------------------------------------
void SampleRenderer::OnRender
(
	ID3D11Device* pDevice, 
	ID3D11DeviceContext* pImmediateContext, 
	double time,
	float  elapsedTime
)
{
	// r[svZ
    mView = DemoApp::GetInstance()->GetCamera()->GetView();
	
	// xNgvZ
    mViewDir = DemoApp::GetInstance()->GetCamera()->GetDirection() - DemoApp::GetInstance()->GetCamera()->GetPosition();

	// ftHg̃_[^[Qbgr[擾
	ID3D11RenderTargetView* pRTV = DemoApp::GetInstance()->GetDefaultRenderTargetView();
    ID3D11DepthStencilView* pDSV = DemoApp::GetInstance()->GetDefaultDepthStencilView();

	// ftHg̃_[^[Qbgr[ɐ؂ւ
	pImmediateContext->OMSetRenderTargets( 1, &pRTV, pDSV );
	pImmediateContext->ClearRenderTargetView( pRTV, DemoApp::GetInstance()->GetDefaultClearColor() );
	pImmediateContext->ClearDepthStencilView( pDSV, D3D11_CLEAR_DEPTH, 1.0f, 0 );

	// VF[_ݒ
	ID3D11VertexShader *pVS = mShaders.GetVS( 0 );
	ID3D11PixelShader  *pPS = mShaders.GetPS( 0 );
	pImmediateContext->VSSetShader( pVS, NULL, 0 );
	pImmediateContext->PSSetShader( pPS, NULL, 0 );

	// 萔obt@ݒ
	D3DXMatrixIdentity( &mWorld );
	ModelUtil::SetMatrix( pImmediateContext, mWorld, mView, mProjection );
    ModelUtil::SetVector( pImmediateContext, mLightDir, mViewDir );
    SetLightColor( pImmediateContext, mLightAmbient, mLightDiffuse, mLightSpecular );

	// f`
	mSaturn.Draw( pImmediateContext );

    // ftHgVF[_ݒ
    pVS = ModelUtil::GetDefaultVertexShader();
    pPS = ModelUtil::GetDefaultPixelShader();
  	pImmediateContext->VSSetShader( pVS, NULL, 0 );
	pImmediateContext->PSSetShader( pPS, NULL, 0 );

	// [hsݒ
    D3DXMatrixTranslation( &mWorld, 0.0f, -1.0f, 0.0f );

	// s]
	ModelUtil::SetMatrix( pImmediateContext, mWorld, mView, mProjection );
	
	// nʂ`
	mGround.Draw( pImmediateContext );

}

//-----------------------------------------------------------------------
//! @brief xNgobt@̐ݒ
//-----------------------------------------------------------------------
void SampleRenderer::SetLightColor
(
    ID3D11DeviceContext* pImmediateContext,
    const D3DXVECTOR3 &lightAmbient,
    const D3DXVECTOR3 &lightDiffuse,
	const D3DXVECTOR3 &lightSpecular
)
{
    LightColorBuffer buffer;
    memset( &buffer, 0, sizeof( LightColorBuffer ) );
    buffer.LightAmbient = lightAmbient;
    buffer.LightDiffuse = lightDiffuse;
	buffer.LightSpecular = lightSpecular;
  
	// slot:2̒萔obt@ɃxNgݒ
    pImmediateContext->UpdateSubresource( mpLightColorBuffer, 0, NULL, &buffer, 0, 0 );
    pImmediateContext->PSSetConstantBuffers( 3, 1, &mpLightColorBuffer );
}

} // namespace SampleProgram
} // namespace Asura
